import * as i0 from '@angular/core';
import { Injectable, Optional, Inject, APP_INITIALIZER, NgModule, SkipSelf } from '@angular/core';
import { DOCUMENT } from '@angular/common';

class ThemingOptions {
}

class Theming {
    constructor(_options, _rendererFactory, _document) {
        this._options = _options;
        this._rendererFactory = _rendererFactory;
        this._document = _document;
        // Readonly data.
        this._attribute = 'data-theme';
        this._renderer = this._rendererFactory.createRenderer(null, null);
        // State.
        this._currentTheme = null;
        this.setCssVariables();
        this.setFirstTheme();
    }
    get all() {
        return this._options?.themes ?? [];
    }
    get current() {
        return this._currentTheme ?? null;
    }
    set(name) {
        const theme = this.all?.find(theme => theme?.name === name);
        if (!theme) {
            throw new Error('Invalid theme.');
        }
        const document = this._document.documentElement;
        if (!document) {
            return;
        }
        this._renderer.setAttribute(document, this._attribute, name);
        this._currentTheme = theme;
    }
    setCssVariables() {
        const style = this._renderer.createElement('style');
        this._renderer.setAttribute(style, 'id', 'theming-style');
        const variables = this.getCssVariables();
        this._renderer.setProperty(style, 'innerHTML', variables);
        const head = this._document?.head ?? null;
        if (!head) {
            return;
        }
        this._renderer.appendChild(this._document?.head, style);
    }
    getCssVariables() {
        return this.all
            .map(theme => `:root[${this._attribute}="${theme?.name}"] {${this.getThemeContents(theme)}}`)
            .join('\n');
    }
    getThemeContents(theme) {
        const properties = this.extractThemeProperties(theme);
        const themeData = this.formatThemeData(properties);
        return Object.keys(themeData)
            .map(key => `${key}: ${themeData?.[key]};`)
            .join(' ');
    }
    formatThemeData(data, scope = null) {
        return Object.keys(data).reduce((previous, current) => {
            const value = data?.[current];
            if (typeof value === 'object' && value !== null) {
                const token = this.getFormattedToken(current, scope);
                const reformatted = this.formatThemeData(value, token);
                return { ...previous, ...reformatted };
            }
            const token = this.getFormattedToken(current, scope);
            return { ...previous, [`--${token}`]: value };
        }, {});
    }
    setFirstTheme() {
        const theme = this.all?.[0] ?? null;
        if (!theme) {
            throw new Error('No theme found.');
        }
        const themeName = theme?.name ?? null;
        return this.set(themeName);
    }
    getFormattedToken(key, scope = null) {
        const result = key.replace(/([A-Z])/g, ' $1');
        const current = result.split(' ').join('-').toLowerCase();
        return [scope, current].filter(item => !!item).join('-');
    }
    extractThemeProperties(theme) {
        // Note: Do not include useless properties like the theme.
        const { name, ...rest } = theme;
        return rest;
    }
}
Theming.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: Theming, deps: [{ token: ThemingOptions }, { token: i0.RendererFactory2 }, { token: DOCUMENT, optional: true }], target: i0.ɵɵFactoryTarget.Injectable });
Theming.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: Theming });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: Theming, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: ThemingOptions }, { type: i0.RendererFactory2 }, { type: Document, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [DOCUMENT]
                }] }]; } });

class ThemingModule {
    constructor(_parent) {
        this._parent = _parent;
        if (this._parent) {
            throw new Error('ThemingModule is already loaded; Import it in the core module only.');
        }
    }
    static forRoot(options) {
        return {
            ngModule: ThemingModule,
            providers: [
                Theming,
                { provide: APP_INITIALIZER, deps: [Theming], useFactory: initializeTheme, multi: true },
                { provide: ThemingOptions, useValue: options }
            ]
        };
    }
}
ThemingModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: ThemingModule, deps: [{ token: ThemingModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule });
ThemingModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.0.3", ngImport: i0, type: ThemingModule });
ThemingModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: ThemingModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.0.3", ngImport: i0, type: ThemingModule, decorators: [{
            type: NgModule,
            args: [{}]
        }], ctorParameters: function () { return [{ type: ThemingModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }]; } });
function initializeTheme(_) {
    // Note: We just need to initialize the theme.
    return () => ({});
}

/**
 * Generated bundle index. Do not edit.
 */

export { Theming, ThemingModule, ThemingOptions, initializeTheme };
//# sourceMappingURL=bravobit-bb-foundation-theming.mjs.map
